import unittest
import skrf as rf
import os

from skrf import network2 as n2

def get_abs_file_path(filename):
    test_dir = os.path.dirname(os.path.abspath(__file__))+'/'
    return os.path.join(test_dir, filename)

class Network2TestCase(unittest.TestCase):
    def setUp(self):
        self.test_dir = os.path.dirname(os.path.abspath(__file__))+'/'
        self.freq = rf.Frequency(1,10,101,'ghz')
        self.s_2port = rf.rand_c(len(self.freq),2,2)
        self.s_1port = rf.rand_c(len(self.freq),1,1)
        self.n = n2.Network(frequency=self.freq, s=self.s_2port, z0=50)


    def get_abs_file_path(filename):
        return os.path.join(self.test_dir, filename)

    ## inits

    #def test_init_empty(self):
    #    n2.Network()

    def test_init_from_s(self):
        n2.Network(frequency=self.freq, s=self.s_2port, z0=50)

    def test_init_from_z(self):
        n2.Network(frequency=self.freq, z=self.s_2port, z0=50)

    def test_init_from_y(self):
        n2.Network(frequency=self.freq, y=self.s_2port, z0=50)

    def test_init_from_ntwkv1(self):
        n2.Network.from_ntwkv1(rf.data.ring_slot)

    ## existence of parameters
    def test_z(self):
        self.n.z

    def test_z(self):
        self.n.y

    def test_z(self):
        self.n.y

    def test_z(self):
        self.n.s_time

    ## existence of parameters
    def test_db(self):
        self.n.s.db

    def test_db10(self):
        self.n.s.db10

    def test_db20(self):
        self.n.s.db20

    def test_mag(self):
        self.n.s.mag

    def test_deg(self):
        self.n.s.deg

    def test_rad(self):
        self.n.s.rad

    def test_re(self):
        self.n.s.re

    def test_im(self):
        self.n.s.im


    def test_call(self):
        for i,j in self.n.port_tuples:
            sij = self.n(i,j)


    def test_windowed(self):
        self.n.windowed()

    def test_string_slicing(self):

        n=self.n['4-7ghz']



import skrf as rf
import unittest
from numpy import log, pi
from numpy.testing import assert_equal, run_module_suite, assert_almost_equal


class TestUnitConversions(unittest.TestCase):
    """
    Test unit-conversion functions
    """

    def setUp(self):
        pass        

    def test_np_2_db(self):
        """
        Test Np to dB conversion with:
            1 [Np] = 20/ln(10) [dB]
        """
        assert_almost_equal(rf.np_2_db(1), 20/log(10))

    def test_db_2_np(self):
        """
        Test dB to Np conversion with: 
            1 [dB] = ln(10)/20 [Np]
        """
        assert_almost_equal(rf.db_2_np(1), log(10)/20)
        
    def test_mag_2_db(self):
        """
        Test magnitude to dB conversion with:
            2 -> +3 dB
        """
        assert_almost_equal(rf.mag_2_db10(2), 10*log(2)/log(10))


    def test_feet_2_meter(self):
        """
        Test feet to meter length conversion
        """
        assert_almost_equal(rf.feet_2_meter(0.01), 0.003048)
        assert_almost_equal(rf.feet_2_meter(1), 0.3048)
        

    def test_meter_2_feet(self):
        """
        Test meter to feet length conversion
        """
        assert_almost_equal(rf.meter_2_feet(0.01), 0.0328084)
        assert_almost_equal(rf.meter_2_feet(1), 3.28084)


    def test_db_per_100feet_2_db_per_100meter(self):
        """
        Test attenuation unit conversion dB/100feet to dB/100m
        """
        assert_almost_equal(rf.db_per_100feet_2_db_per_100meter(), rf.meter_2_feet(), decimal=2)
        assert_almost_equal(rf.db_per_100feet_2_db_per_100meter(2.5), 8.2, decimal=2)
        assert_almost_equal(rf.db_per_100feet_2_db_per_100meter(0.28), 0.92, decimal=2)

if __name__ == "__main__":
    # Launch all tests
    run_module_suite()
    



# -*- coding: utf-8 -*-
"""
.. module:: skrf.tlineFunctions
===============================================
tlineFunctions (:mod:`skrf.tlineFunctions`)
===============================================

This module provides functions related to transmission line theory.

Impedance and Reflection Coefficient
--------------------------------------
These functions relate basic transmission line quantities such as
characteristic impedance, input impedance, reflection coefficient, etc.
Each function has two names. One is a long-winded but readable name and
the other is a short-hand variable-like names. Below is a table relating
these two names with each other as well as common mathematical symbols.

====================  ======================  ================================
Symbol                Variable Name           Long Name
====================  ======================  ================================
:math:`Z_l`           z_l                     load_impedance
:math:`Z_{in}`        z_in                    input_impedance
:math:`\\Gamma_0`      Gamma_0                 reflection_coefficient
:math:`\\Gamma_{in}`   Gamma_in                reflection_coefficient_at_theta
:math:`\\theta`        theta                   electrical_length
====================  ======================  ================================

There may be a bit of confusion about the difference between the load
impedance the input impedance. This is because the load impedance **is**
the input impedance at the load. An illustration may provide some
useful reference.

Below is a (bad) illustration of a section of uniform transmission line
of characteristic impedance :math:`Z_0`, and electrical length
:math:`\\theta`. The line is terminated on the right with some
load impedance, :math:`Z_l`. The input impedance :math:`Z_{in}` and
input reflection coefficient :math:`\\Gamma_{in}` are
looking in towards the load from the distance :math:`\\theta` from the
load.

.. math::
        Z_0, \\theta

        \\text{o===============o=}[Z_l]

        \\to\\qquad\\qquad\\qquad\\quad\\qquad \\qquad \\to \\qquad \\quad

        Z_{in},\\Gamma_{in}\\qquad\\qquad\\qquad\\qquad\\quad Z_l,\\Gamma_0

So, to clarify the confusion,

.. math::
        Z_{in}= Z_{l},\\qquad\\qquad
        \\Gamma_{in}=\\Gamma_l \\text{ at }  \\theta=0


Short names
+++++++++++++
.. autosummary::
        :toctree: generated/

        theta
        zl_2_Gamma0
        Gamma0_2_zl
        zl_2_zin
        zl_2_Gamma_in
        Gamma0_2_Gamma_in
        Gamma0_2_zin

Long-names
++++++++++++++
.. autosummary::
        :toctree: generated/

        distance_2_electrical_length
        electrical_length_2_distance

        reflection_coefficient_at_theta
        reflection_coefficient_2_input_impedance
        reflection_coefficient_2_input_impedance_at_theta

        input_impedance_at_theta
        load_impedance_2_reflection_coefficient
        load_impedance_2_reflection_coefficient_at_theta




Distributed Circuit and Wave Quantities
----------------------------------------
.. autosummary::
        :toctree: generated/

        distributed_circuit_2_propagation_impedance
        propagation_impedance_2_distributed_circuit

Transmission Line Physics
---------------------------------
.. autosummary::
        :toctree: generated/

        skin_depth
        surface_resistivity
"""

import numpy as npy
from numpy import pi, sqrt, exp, array, imag, real

from scipy.constants import mu_0
from . import mathFunctions as mf

INF = 1e99
ONE = 1.0 + 1/1e14


def skin_depth(f, rho, mu_r):
    """
    Skin depth for a material.

    see www.microwaves101.com for more info.

    Parameters
    ----------
    f : number or array-like
            frequency, in Hz
    rho : number of array-like
            bulk resistivity of material, in ohm*m
    mu_r : number or array-like
            relative permeability of material

    Returns
    -------
    skin depth : number or array-like
            the skin depth, in m

    References
    ----------
    .. [1] http://en.wikipedia.org/wiki/Skin_effect

    """
    return sqrt(rho/(pi*f*mu_r*mu_0))


def surface_resistivity(f, rho, mu_r):
    """
    Surface resistivity.

    see www.microwaves101.com for more info.

    Parameters
    ----------
    f : number or array-like
            frequency, in Hz
    rho : number or array-like
            bulk resistivity of material, in ohm*m
    mu_r : number or array-like
            relative permeability of material

    Returns
    -------
    surface resistivity: ohms/square


    """
    return rho/skin_depth(rho=rho, f=f, mu_r=mu_r)


def distributed_circuit_2_propagation_impedance(distributed_admittance,
        distributed_impedance):
    r"""
    Convert distrubuted circuit values to wave quantities.

    This converts complex distributed impedance and admittance to
    propagation constant and characteristic impedance. The relation is

    .. math::
            Z_0 = \\sqrt{ \\frac{Z^{'}}{Y^{'}}}
            \\quad\\quad
            \\gamma = \\sqrt{ Z^{'}  Y^{'}}

    Parameters
    ----------
    distributed_admittance : number, array-like
            distributed admittance
    distributed_impedance :  number, array-like
            distributed impedance

    Returns
    -------
    propagation_constant : number, array-like
            distributed impedance
    characteristic_impedance : number, array-like
            distributed impedance

    See Also
    --------
            propagation_impedance_2_distributed_circuit : opposite conversion
    """
    propagation_constant = \
            sqrt(distributed_impedance*distributed_admittance)
    characteristic_impedance = \
            sqrt(distributed_impedance/distributed_admittance)
    return (propagation_constant, characteristic_impedance)


def propagation_impedance_2_distributed_circuit(propagation_constant,
        characteristic_impedance):
    r"""
    Convert wave quantities to distributed circuit values.

    Convert complex propagation constant and characteristic impedance
    to distributed impedance and admittance. The relation is,

    .. math::
            Z^{'} = \\gamma  Z_0 \\quad\\quad
            Y^{'} = \\frac{\\gamma}{Z_0}

    Parameters
    ----------
    propagation_constant : number, array-like
            distributed impedance
    characteristic_impedance : number, array-like
            distributed impedance

    Returns
    -------
    distributed_admittance : number, array-like
            distributed admittance
    distributed_impedance :  number, array-like
            distributed impedance


    See Also
    --------
            distributed_circuit_2_propagation_impedance : opposite conversion
    """
    distributed_admittance = propagation_constant/characteristic_impedance
    distributed_impedance = propagation_constant*characteristic_impedance
    return (distributed_admittance, distributed_impedance)


def electrical_length(gamma, f, d, deg=False):
    r"""
    Calculates the electrical length of a section of transmission line.

    .. math::
            \\theta = \\gamma(f) \\cdot d

    Parameters
    ----------
    gamma : number, array-like or function
            propagation constant. See Notes.
            If passed as a function, takes frequency in Hz as a sole argument. 
    f : number or array-like
            frequency at which to calculate
    d : number or array-like
            length of line, in meters
    deg : Boolean
            return in degrees or not.

    Returns
    -------
    theta :  number or array-like
            electrical length in radians or degrees, depending on  value of
            deg.

    See Also
    --------
            electrical_length_2_distance : opposite conversion

    Notes
    -----
    the convention has been chosen that forward propagation is
    represented by the positive imaginary part of the value returned by
    the gamma function
    """
    # if gamma is not a function, create a dummy function which return gamma
    if not callable(gamma):
        _gamma = gamma
        def gamma(f0): return _gamma

    # typecast to a 1D array
    f = array(f, dtype=float).reshape(-1)
    d = array(d, dtype=float).reshape(-1)

    if deg == False:
        return  gamma(f)*d
    elif deg == True:
        return  mf.radian_2_degree(gamma(f)*d )


def electrical_length_2_distance(theta, gamma, f0, deg=True):
    r"""
    Convert electrical length to a physical distance.

    .. math::
            d = \\frac{\\theta}{\\gamma(f_0)}

    Parameters
    ----------
    theta : number or array-like
            electical length. units depend on `deg` option
    gamma : number, array-like or function
            propagation constant. See Notes.
            If passed as a function, takes frequency in Hz as a sole argument.
    f0 : number or array-like
            frequency at which to calculate gamma
    deg : Boolean
            return in degrees or not.

    Returns
    -------
    d : number or array-like (real)
        physical distance in m

    Notes
    -----
    the convention has been chosen that forward propagation is
    represented by the positive imaginary part of the value returned by
    the gamma function

    See Also
    --------
            distance_2_electrical_length: opposite conversion
    """
    # if gamma is not a function, create a dummy function which return gamma
    if not callable(gamma):
        _gamma = gamma
        def gamma(f0): return _gamma

    if deg:
        theta = mf.degree_2_radian(theta)
    return real(theta / gamma(f0))


def load_impedance_2_reflection_coefficient(z0, zl):
    r"""
    Reflection coefficient from a load impedance.    

    Return the reflection coefficient for a given load impedance, and
    characteristic impedance.

    For a transmission line of characteristic impedance :math:`Z_0`
    terminated with load impedance :math:`Z_l`, the complex reflection
    coefficient is given by,

    .. math::
            \\Gamma = \\frac {Z_l - Z_0}{Z_l + Z_0}


    Parameters
    ----------
    z0 :  number or array-like
            characteristic impedance
    zl :  number or array-like
            load impedance (aka input impedance)


    Returns
    -------
    gamma : number or array-like
            reflection coefficient

    See Also
    --------
            Gamma0_2_zl : reflection coefficient to load impedance


    Notes
    -----
            inputs are typecasted to 1D complex array
    """
    # typecast to a complex 1D array. this makes everything easier
    z0 = array(z0, dtype=complex).reshape(-1)
    zl = array(zl, dtype=complex).reshape(-1)

    # handle singularity  by numerically representing inf as big number
    zl[(zl == npy.inf)] = INF

    return ((zl - z0)/(zl + z0))


def reflection_coefficient_2_input_impedance(z0, Gamma):
    r"""
    Input impedance from a load reflection coefficient.

    Calculate the input impedance given a reflection coefficient and
    characteristic impedance.

    .. math::
            Z_0 (\\frac {1 + \\Gamma}{1-\\Gamma})


    Parameters
    ----------
    Gamma : number or array-like
            complex reflection coefficient
    z0 : number or array-like
            characteristic impedance

    Returns
    -------
    zin : number or array-like
            input impedance

    """
    # typecast to a complex 1D array. this makes everything easier
    Gamma = array(Gamma, dtype=complex).reshape(-1)
    z0 = array(z0, dtype=complex).reshape(-1)

    # handle singularity by numerically representing inf as close to 1
    Gamma[(Gamma == 1)] = ONE

    return z0*((1.0 + Gamma)/(1.0 - Gamma))


def reflection_coefficient_at_theta(Gamma0, theta):
    r"""
    Reflection coefficient at a given electrical length.

    .. math::
            \\Gamma_{in} = \\Gamma_0 e^{-2 \\theta}

    Parameters
    ----------
    Gamma0 : number or array-like
            reflection coefficient at theta=0
    theta : number or array-like
            electrical length (may be complex)

    Returns
    -------
    Gamma_in : number or array-like
            input reflection coefficient

    """
    Gamma0 = array(Gamma0, dtype=complex).reshape(-1)
    theta = array(theta, dtype=complex).reshape(-1)
    return Gamma0 * exp(-2*theta)


def input_impedance_at_theta(z0, zl, theta):
    """
    Input impedance from load impedance at a given electrical length.

    Input impedance of load impedance zl at a given electrical length,
    given characteristic impedance z0.

    Parameters
    ----------
    z0 : number or array-like
        characteristic impedance
    zl : number or array-like
        load impedance
    theta : number or array-like
        electrical length of the line (may be complex)

    Returns
    -------
    zin : number or array-like
        input impedance at theta

    """
    Gamma0 = load_impedance_2_reflection_coefficient(z0=z0, zl=zl)
    Gamma_in = reflection_coefficient_at_theta(Gamma0=Gamma0, theta=theta)
    return reflection_coefficient_2_input_impedance(z0=z0, Gamma=Gamma_in)


def load_impedance_2_reflection_coefficient_at_theta(z0, zl, theta):
    """
    Reflection coefficient of load at a given electrical length.

    Reflection coefficient of load impedance zl at a given electrical length,
    given characteristic impedance z0.

    Parameters
    ----------
    z0 : number or array-like
        characteristic impedance.
    zl : number or array-like
        load impedance
    theta : number or array-like
        electrical length of the line (may be complex).

    Returns
    -------
    Gamma_in : number or array-like
        input reflection coefficient at theta

    """
    Gamma0 = load_impedance_2_reflection_coefficient(z0=z0, zl=zl)
    Gamma_in = reflection_coefficient_at_theta(Gamma0=Gamma0, theta=theta)
    return Gamma_in


def reflection_coefficient_2_input_impedance_at_theta(z0, Gamma0, theta):
    """
    Input impedance from load reflection coefficient at a given electrical length.

    Calculate the input impedance at electrical length theta, given a
    reflection coefficient and characteristic impedance of the medium.

    Parameters
    ----------
    z0 : number or array-like
        characteristic impedance.
    Gamma: number or array-like
        reflection coefficient
    theta: number or array-like
        electrical length of the line, (may be complex)

    Returns
    -------
    zin: number or array-like
        input impedance at theta

    """
    Gamma_in = reflection_coefficient_at_theta(Gamma0=Gamma0, theta=theta)
    zin = reflection_coefficient_2_input_impedance(z0=z0, Gamma=Gamma_in)
    return zin


def reflection_coefficient_2_propagation_constant(Gamma_in, Gamma_l, d):
    r"""
    Propagation constant from line input and load reflection coefficients.

    Calculate the propagation constant of a line of length d, given the
    reflection coefficient and characteristic impedance of the medium.

        .. math::
            \\Gamma_{in} = \\Gamma_l e^{-2 j \\gamma \\cdot d}
            \\to \\gamma = -\\frac{1}{2 d} \\ln \\left ( \\frac{ \\Gamma_{in} }{ \\Gamma_l } \\right )

    Parameters
    ----------
    Gamma_in :  number or array-like
            input reflection coefficient
    Gamma_l :  number or array-like
            load reflection coefficient
    d : number or array-like
            length of line, in meters

    Returns
    -------
    gamma : number (complex) or array-like
            propagation constant (see notes)

    Notes
    -----
    the convention has been chosen that forward propagation is
    represented by the positive imaginary part of gamma

    """
    gamma = -1/(2*d) * npy.log(Gamma_in/Gamma_l)
    # the imaginary part of gamma (=beta) cannot be negative with the given
    # definition of gamma. Thus one should take the first modulo positive value
    gamma.imag = gamma.imag % (pi/d)

    return gamma


def Gamma0_2_swr(Gamma0):
    r"""
    Return Standing Wave Ratio (SWR) for a given reflection coefficient.

    Standing Wave Ratio value is defined by:

        .. math::
            VSWR = \\frac{1 + |\\Gamma_0|}{1 - |\Gamma_0|}

    Parameters
    ----------
    Gamma0 : number or array-like
        Reflection coefficient

    Returns
    -------
    swr : number or array-like
        Standing Wave Ratio.

    """
    return (1 + npy.abs(Gamma0)) / (1 - npy.abs(Gamma0))


def zl_2_swr(z0, zl):
    r"""
    Return Standing Wave Ratio (SWR) for a given load impedance.

    Standing Wave Ratio value is defined by:

        .. math::
            VSWR = \\frac{1 + |\\Gamma|}{1 - |\Gamma|}
            where
            \\Gamma = \\frac{Z_L - Z_0}{Z_L + Z_0}

    Parameters
    ----------
    z0 : number or array-like
        line characteristic impedance [Ohm]
    zl : number or array-like
        load impedance [Ohm]

    Returns
    -------
    swr : number or array-like
        Standing Wave Ratio.

    """
    Gamma0 = load_impedance_2_reflection_coefficient(z0, zl)
    return Gamma0_2_swr(Gamma0)


def voltage_current_propagation(v1, i1, z0, theta):
    """
    Propagate voltage and current on electrical length theta of a transmission line.

    Give voltage v2 and current i1 at theta, given voltage v1 
    and current i1 (at theta=0) and given characteristic parameters gamma and z0.
    
    
      i1                     i2
    ○-->---------------------->--○
    
    v1         gamma,z0         v2
    
    ○----------------------------○
    
    <------------ d ------------->
    
    0                         theta
    
    Use (inverse) ABCD parameters of a transmission line. 
    
    Parameters
    ----------
    v1 : array-like (nfreqs,)
        total voltage at z=0
    i1 : array-like (nfreqs,)
        total current at z=0, directed toward the transmission line
    z0: array-like (nfreqs,)
        characteristic impedance
    theta : number or array-like (nfreq, ntheta)
        electrical length of the line (may be complex).
    
    Return
    ------
    v2 : array-like (nfreqs, ntheta)
        total voltage at z=d
    i2 : array-like (nfreqs, ndtheta
        total current at z=d, directed outtoward the transmission line
    """
    # outer product by broadcasting of the electrical length
    #theta = gamma[:, npy.newaxis] * d  # (nbfreqs x nbd)
    # ABCD parameters of a transmission line (gamma, z0)
    A = npy.cosh(theta)
    B = z0*npy.sinh(theta)
    C = npy.sinh(theta)/z0
    D = npy.cosh(theta)
    # transpose and de-transpose operations are necessary 
    # for linalg.inv to inverse square matrices
    ABCD = npy.array([[A, B],[C, D]]).transpose()   
    inv_ABCD = npy.linalg.inv(ABCD).transpose()
    
    v2 = inv_ABCD[0,0] * v1 + inv_ABCD[0,1] * i1
    i2 = inv_ABCD[1,0] * v1 + inv_ABCD[1,1] * i1
    return v2, i2


def zl_2_total_loss(z0, zl, theta):
    '''
    Total loss of terminated transmission line (in natural unit)
    
    The total loss expressed in terms of the load impedance is [#]_:

        .. math::
            TL = \\frac{R_{in}}{R_L} \\left| \\cosh \\theta  + \\frac{Z_L}{Z_0} \\sinh\\theta \\right|^2
    
    Parameters
    ----------
    z0 : number or array-like
        characteristic impedance.
    zl : number or array-like
        load impedance
    theta : number or array-like
        electrical length of the line (may be complex).

    Returns
    -------
    total_loss: number or array-like
        total loss in natural unit
        
    References
    ----------
    .. [#]: Steve Stearns (K6OIK), Transmission Line Power Paradoxand Its Resolution. ARRL PacificonAntenna Seminar, Santa Clara, CA, October 10-12, 2014. https://www.fars.k6ya.org/docs/K6OIK-A_Transmission_Line_Power_Paradox_and_Its_Resolution.pdf
    
    
    '''
    Rin = npy.real(zl_2_zin(z0, zl, theta))
    total_loss = Rin/npy.real(zl)*npy.abs(npy.cosh(theta) + zl/z0*npy.sinh(theta))**2
    return total_loss


# short hand convenience.
# admittedly these follow no logical naming scheme, but they closely
# correspond to common symbolic conventions, and are convenient
theta = electrical_length
distance_2_electrical_length = electrical_length

zl_2_Gamma0 = load_impedance_2_reflection_coefficient
Gamma0_2_zl = reflection_coefficient_2_input_impedance

zl_2_zin = input_impedance_at_theta
zl_2_Gamma_in = load_impedance_2_reflection_coefficient_at_theta

Gamma0_2_Gamma_in = reflection_coefficient_at_theta
Gamma0_2_zin = reflection_coefficient_2_input_impedance_at_theta








import unittest
import os
import numpy as npy

import skrf as rf

from nose.plugins.skip import SkipTest
from skrf.util import suppress_warning_decorator

class AgilentCSVTestCase(unittest.TestCase):
    ''' 
    AgilentCSVTestCase tests the IO of agilent style CSV files
    '''
    def setUp(self):
        ''' 
        Sets up the test directory and the initializes the members.
        This method gets the currect file path to this file, then gets the file
        name for pna_csv_reim.csv file, then reads it in.
        '''
        self.test_dir = os.path.dirname(os.path.abspath(__file__))+'/'
        filename = os.path.join(self.test_dir, 'pna_csv_reim.csv')
        self.acsv = rf.AgilentCSV(filename)

    def test_columns(self):
        ''' 
        This tests reading of columns from the test file.
        '''
        self.assertEqual(self.acsv.columns, ['Freq(Hz)', '"A,1"(REAL)',
                                             '"A,1"(IMAG)', '"R1,1"(REAL)',
                                             '"R1,1"(IMAG)'])
    @SkipTest # unicode error with carrage returns for p3 vs p2
    def test_comments(self):
        ''' 
        This tests reading of comment lines in the test file.
        '''
        self.assertEqual(self.acsv.comments.strip('\r'), 'this is a comment\nline\n')

    def test_data(self):
        '''
        This tests reading in of the data of the test file.
        '''
        self.assertTrue((self.acsv.data ==
                         npy.array([[750000000000, 1, 2, 3, 4],
                                    [1100000000000, 5, 6, 7,8],
                                   ])).all())

    def test_frequency(self):
        ''' 
        This tests the reading of frequency from the test file
        '''
        self.assertEqual(self.acsv.frequency, rf.F(750e9, 1100e9, 2, 'hz'))

    @suppress_warning_decorator("CSV format unrecognized")
    def test_networks(self):
        '''
        This only tests for execution, not accuracy
        '''
        a = self.acsv.networks

    def test_scalar_networks(self):
        '''
        This only tests for execution, not accuracy
        '''
        a = self.acsv.scalar_networks




'''
.. module:: skrf.media.rectangularWaveguide

================================================================
rectangularWaveguide (:mod:`skrf.media.rectangularWaveguide`)
================================================================

Represents a single mode of a homogeneously filled rectangular
waveguide of cross-section `a` x `b`. The mode is determined by
`mode-type` (`'te'` or `'tm'`) and mode indices ( `m` and `n` ).


====================================  =============  ===============
Quantity                              Symbol         Variable
====================================  =============  ===============
Characteristic Wave Number            :math:`k_0`    :attr:`k0`
Cut-off Wave Number                   :math:`k_c`    :attr:`kc`
Longitudinal Wave Number              :math:`k_z`    :attr:`gamma`
Transverse Wave Number (a)            :math:`k_x`    :attr:`kx`
Transverse Wave Number (b)            :math:`k_y`    :attr:`ky`
Characteristic Impedance              :math:`z_0`    :attr:`Z0`
====================================  =============  ===============

.. autosummary::
   :toctree: generated/

   RectangularWaveguide

'''
from scipy.constants import  epsilon_0, mu_0,pi,c
from numpy import sqrt, exp, sinc,where
import numpy as npy
from .media import Media
from ..data import materials
from ..tlineFunctions import skin_depth

from .freespace import Freespace

class RectangularWaveguide(Media):
    '''
    A single mode of a homogeneously filled rectangular waveguide

    Parameters
    ----------
    frequency : :class:`~skrf.frequency.Frequency` object
            frequency band of this transmission line medium
    z0 : number, array-like, or None
        the port impedance for media. Only needed if  its different
        from the characterisitc impedance of the transmission
        line. if z0 is None then will default to Z0
    a : number
            width of waveguide, in meters.
    b : number
            height of waveguide, in meters. If `None` defaults to a/2
    mode_type : ['te','tm']
            mode type, transverse electric (te) or transverse magnetic
            (tm) to-z. where z is direction of propagation
    m : int
            mode index in 'a'-direction
    n : int
            mode index in 'b'-direction
    ep_r : number, array-like,
            filling material's relative permittivity
    mu_r : number, array-like
            filling material's relative permeability
    rho : number, array-like, string
        resistivity (ohm-m) of the conductor walls. If array-like
        must be same length as frequency. if str, it must be a key in
        `skrf.data.materials`.
    roughness : number, or array-like
        surface roughness of the conductor walls in units of RMS
        deviation from surface

    *args,**kwargs : arguments, keywrod arguments
            passed to :class:`~skrf.media.media.Media`'s constructor
            (:func:`~skrf.media.media.Media.__init__`


    Examples
    ------------
    Most common usage is standard aspect ratio (2:1) dominant
    mode, TE10 mode of wr10 waveguide can be constructed by

    >>> freq = rf.Frequency(75,110,101,'ghz')
    >>> rf.RectangularWaveguide(freq,a= 100*mil)
    '''
    def __init__(self, frequency=None, z0=None, a=1, b=None,
                 mode_type = 'te', m=1, n=0, ep_r=1, mu_r=1, rho=None, 
                 roughness=None, *args, **kwargs):
        
        Media.__init__(self, frequency=frequency,z0=z0)
        
        if b is None:
            b = a/2.
        if mode_type.lower() not in ['te','tm']:
            raise ValueError('mode_type must be either \'te\' or \'tm\'')

        
        self.a = a
        self.b = b
        self.mode_type = mode_type
        self.m = m
        self.n = n
        self.ep_r = ep_r
        self.mu_r = mu_r
        self.rho = rho
        self.roughness = roughness
        

    def __str__(self):
        f=self.frequency
        output =  \
                'Rectangular Waveguide Media.  %i-%i %s.  %i points'%\
                (f.f_scaled[0],f.f_scaled[-1],f.unit, f.npoints) + \
                '\n a= %.2em, b= %.2em'% \
                (self.a,self.b)
        return output

    def __repr__(self):
        return self.__str__()

    
    @classmethod
    def from_Z0(cls,frequency, Z0,f, m=1,n=0,ep_r=1, mu_r=1, **kw):
        '''
        Initialize from specfied impedance at a given frequency.
        
        Parameters
        -------------
        frequency : Frequency Object
        Z0 : number /array
            scharacteristic impedance to create at `f`
        f : number 
            frequency (in Hz) that the resultant waveguide has Z0=Z0
        '''
        if n !=0: 
            raise NotImplemented()
        
        
        mu = mu_0*mu_r
        ep = epsilon_0*ep_r
        w = 2*pi*f
        a =pi/(w*mu) * 1./sqrt((1/(Z0*1j)**2+ep/mu))
        
        kw.update(dict(frequency=frequency,a=a, m=m, n=n, ep_r=ep_r, mu_r=mu_r))
        
        return cls(**kw)
    
    @property
    def ep(self):
        '''
        The permativity of the filling material

        Returns
        -------
        ep : number
                filling material's relative permittivity
        '''
        return self.ep_r * epsilon_0

    @property
    def mu(self):
        '''
        The permeability of the filling material

        Returns
        -------
        mu : number
                filling material's relative permeability

        '''
        return self.mu_r * mu_0

    @property
    def k0(self):
        '''
        Characteristic wave number

        Returns
        -------
        k0 : number
                characteristic wave number
        '''
        return 2*pi*self.frequency.f*sqrt(self.ep * self.mu)

    @property
    def ky(self):
        '''
        Eigen-value in the `b` direction.

        Defined as

        .. math::

                k_y = n \\frac{\pi}{b}

        Returns
        -------
        ky : number
                eigen-value in `b` direction
        '''
        return self.n*pi/self.b

    @property
    def kx(self):
        '''
        Eigen value in the 'a' direction

        Defined as

        .. math::

                k_x = m \\frac{\pi}{a}

        Returns
        -------
        kx : number
                eigen-value in `a` direction
        '''
        return self.m*pi/self.a

    @property
    def kc(self):
        '''
        Cut-off wave number

        Defined as

        .. math::

                k_c = \\sqrt {k_x^2 + k_y^2} = \\sqrt {
                {m \\frac{\pi}{a}}^2 + {n \\frac{\pi}{b}}^2}

        Returns
        -------
        kc : number
                cut-off wavenumber
        '''
        return sqrt( self.kx**2 + self.ky**2)


    @property
    def f_cutoff(self):
        '''
        cutoff frequency for this mode

        .. math::

            max ( \frac{m \cdot v}{2a} , \frac{n \cdot v}{2b})

        where v= sqrt(ep*mu)



        '''
        v = 1/sqrt(self.ep*self.mu)
        if not ( self.m==1 and self.n==0):
            print('f_cutoff not verified as correct for this mode ')
        
        if self.m/self.a > self.n/self.b: 
            return self.m*v/(2*self.a)
        else: 
            return self.n*v/(2*self.b)

    @property
    def f_norm(self):
        '''
        frequency vector normalized to cutoff
        '''
        return self.frequency.f/self.f_cutoff

    @property
    def rho(self):
        '''
        conductivty of sidewalls in ohm*m

        Parameters
        --------------
        val : float, array-like or str
            the conductivity in ohm*m. If array-like must be same length
            as self.frequency. if str, it must be a key in
            `skrf.data.materials`.

        Examples
        ---------
        >>> wg.rho = 2.8e-8
        >>> wg.rho = 2.8e-8 * ones(len(wg.frequency))
        >>> wg.rho = 'al'
        >>> wg.rho = 'aluminum'
        '''
        if self.roughness != None:
            delta = skin_depth(self.frequency.f, self._rho, self.mu_r)
            k_w = 1. +exp(-(delta/(2*self.roughness))**1.6)
            return self._rho*k_w**2

        return self._rho

    @rho.setter
    def rho(self, val):
        if isinstance(val, str):
            self._rho = materials[val.lower()]['resistivity(ohm*m)']
        else:
            self._rho=val

    @property
    def lambda_guide(self):
        '''
        guide wavelength

        the distance in which the phase of the field increases by 2 pi
        '''
        return 2*pi/self.beta

    @property
    def lambda_cutoff(self):
        '''
        cuttoff wavelength

        .. math::

            f_c * v

         where v= sqrt(ep*mu)
        '''
        v = 1/sqrt(self.ep*self.mu)
        return self.f_cutoff*v

    @property
    def gamma(self):
        '''
        The propagation constant (aka Longitudinal wave number)

        Defined as

        .. math::

                k_z = j \\pm \\sqrt {k_0^2 - k_c^2}

        This is.
                * IMAGINARY for propagating modes
                * REAL  for non-propagating modes,

        Returns
        --------
        gamma :  number
                The propagation constant


        '''
        ## haringtons form
        if False:#self.m==1 and self.n==0:
            fs = Freespace(frequency=self.frequency, 
                           ep_r=self.ep_r, 
                           mu_r=self.mu_r)
                           
            
            g= where(self.f_norm>1.,
                     sqrt(1-self.f_norm**(-2))*fs.gamma, # cutton
                 -1j*sqrt(1-self.f_norm**(2))*fs.gamma)# cutoff
        
        else:
            # TODO:  fix this for lossy ep/mu (remove abs?)
            k0,kc = self.k0, self.kc
            g=  1j*sqrt(abs(k0**2 - kc**2)) * (k0>kc) +\
                    sqrt(abs(kc**2- k0**2))*(k0<kc) + \
                    0*(kc==k0) 

        g = g + self.alpha_c *(self.rho is not None)
        
        return g 
        
        
    @property
    def alpha_c(self):
        '''
        Loss due to finite conductivity and roughness of sidewalls

        In units of np/m
        See property `rho` for setting conductivity.

        Effects of finite conductivity are taken from [#]_. If
        :attr:`roughness` is not None, then its effects the conductivity
        by


        .. math::

            \\sigma_c = \\frac{\\sigma}{k_w^2}


        where

        .. math::

            k_w = 1 + e^{(-\\delta/2h)^{1.6}}

            \\delta = \\mbox{skin depth}

            h = \\mbox{surface roughness }


        This is taken from Ansoft HFSS help documents.



        References
        --------------

        .. [#] Chapter 9, (eq 9.8.1) of Electromagnetic Waves and Antennas by Sophocles J. Orfanidis
        http://eceweb1.rutgers.edu/~orfanidi/ewa/
        '''

        if self.rho is None:
            return 0

        a,b,w,ep,rho,f_n = self.a, self.b, self.frequency.w, self.ep, \
            self.rho, self.f_norm



        return 1./b * sqrt( (w*ep)/(2./rho) ) * (1+2.*b/a*(1/f_n)**2)/\
            sqrt(1-(1/f_n)**2)

    @property
    def Z0(self):
        '''
        The characteristic impedance
        '''
        omega = self.frequency.w
        impedance_dict = {'te':   1j*omega*self.mu/(self.gamma),
                          'tm':   -1j*self.gamma/(omega*self.ep),\
                         }

        return impedance_dict[self.mode_type]





# -*- coding: utf-8 -*-
import unittest
import os
import numpy as npy


from skrf.media import DefinedGammaZ0, Media
from skrf.network import Network
from skrf.frequency import Frequency
import skrf


class DefinedGammaZ0TestCase(unittest.TestCase):
    def setUp(self):
        self.files_dir = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'qucs_prj'
            )
        self.dummy_media = DefinedGammaZ0(
            frequency = Frequency(1,100,21,'ghz'),
            gamma=1j,
            z0 = 50 ,
            )

    def test_impedance_mismatch(self):
        '''
        '''
        fname = os.path.join(self.files_dir,\
                'impedanceMismatch,50to25.s2p')
        qucs_ntwk = Network(fname)
        self.dummy_media.frequency = qucs_ntwk.frequency
        skrf_ntwk = self.dummy_media.thru(z0=50)**\
            self.dummy_media.thru(z0=25)

        self.assertEqual(qucs_ntwk, skrf_ntwk)

    def test_resistor(self):
        '''
        '''
        fname = os.path.join(self.files_dir,\
                'resistor,1ohm.s2p')
        qucs_ntwk = Network(fname)
        self.dummy_media.frequency = qucs_ntwk.frequency
        skrf_ntwk = self.dummy_media.resistor(1)
        self.assertEqual(qucs_ntwk, skrf_ntwk)

    def test_capacitor(self):
        '''
        '''
        fname = os.path.join(self.files_dir,\
                'capacitor,p01pF.s2p')
        qucs_ntwk = Network(fname)
        self.dummy_media.frequency = qucs_ntwk.frequency
        skrf_ntwk = self.dummy_media.capacitor(.01e-12)
        self.assertEqual(qucs_ntwk, skrf_ntwk)


    def test_inductor(self):
        '''
        '''
        fname = os.path.join(self.files_dir,\
                'inductor,p1nH.s2p')
        qucs_ntwk = Network(fname)
        self.dummy_media.frequency = qucs_ntwk.frequency
        skrf_ntwk = self.dummy_media.inductor(.1e-9)
        self.assertEqual(qucs_ntwk, skrf_ntwk)


    def test_scalar_gamma_z0_media(self):
        '''
        test ability to create a Media from scalar quanties for gamma/z0
        and change frequency resolution
        '''
        a = DefinedGammaZ0 (Frequency(1,10,101),gamma=1j,z0 = 50)
        self.assertEqual(a.line(1),a.line(1))

        # we should be able to re-sample the media
        a.npoints = 21
        self.assertEqual(len(a.gamma), len(a))
        self.assertEqual(len(a.z0), len(a))
        self.assertEqual(len(a.z0), len(a))


    def test_vector_gamma_z0_media(self):
        '''
        test ability to create a Media from vector quanties for gamma/z0
        '''
        freq = Frequency(1,10,101)
        a = DefinedGammaZ0(freq,
                           gamma = 1j*npy.ones(len(freq)) ,
                           z0 =  50*npy.ones(len(freq)),
                            )


        self.assertEqual(a.line(1),a.line(1))
        with self.assertRaises(NotImplementedError):
            a.npoints=4

    def test_write_csv(self):
        fname = os.path.join(self.files_dir,\
                'out.csv')
        self.dummy_media.write_csv(fname)
        os.remove(fname)


    def test_from_csv(self):
        fname = os.path.join(self.files_dir,\
                'out.csv')
        self.dummy_media.write_csv(fname)
        a_media = DefinedGammaZ0.from_csv(fname)
        self.assertEqual(a_media,self.dummy_media)
        os.remove(fname)


class STwoPortsNetworkTestCase(unittest.TestCase):
    '''
    Check that S parameters of media base elements versus theoretical results.
    '''
    def setUp(self):
        self.dummy_media = DefinedGammaZ0(
            frequency=Frequency(1, 100, 21, 'GHz'),
            gamma=1j,
            z0=50,
            )

    def test_s_series_element(self):
        '''
        Series elements of impedance Z:

        ○---[Z]---○

        ○---------○

        have S matrix of the form:
        [ Z/Z0 / (Z/Z0 + 2)     2/(Z/Z0 + 2) ]
        [ 2/(Z/Z0 + 2)          Z/Z0 / (Z/Z0 + 2) ]
        '''
        R = 1.0  # Ohm
        ntw = self.dummy_media.resistor(R)
        Z0 = self.dummy_media.z0
        S11 = (R/Z0) / (R/Z0 + 2)
        S21 = 2 / (R/Z0 + 2)
        npy.testing.assert_array_almost_equal(ntw.s[:,0,0], S11)
        npy.testing.assert_array_almost_equal(ntw.s[:,0,1], S21)
        npy.testing.assert_array_almost_equal(ntw.s[:,1,0], S21)
        npy.testing.assert_array_almost_equal(ntw.s[:,1,1], S11)

    def test_s_shunt_element(self):
        '''
        Shunt elements of admittance Y:

        ○---------○
            |
           [Y]
            |
        ○---------○

        have S matrix of the form:
        [ -Y Z0 / (Y Z0 + 2)     2/(Y Z0 + 2) ]
        [ 2/(Y Z0 + 2)          Z/Z0 / (Y Z0 + 2) ]
        '''
        R = 1.0  # Ohm
        ntw = self.dummy_media.shunt(self.dummy_media.resistor(R)**self.dummy_media.short())
        Z0 = self.dummy_media.z0
        S11 = -(1/R*Z0) / (1/R*Z0 + 2)
        S21 = 2 / (1/R*Z0 + 2)
        npy.testing.assert_array_almost_equal(ntw.s[:,0,0], S11)
        npy.testing.assert_array_almost_equal(ntw.s[:,0,1], S21)
        npy.testing.assert_array_almost_equal(ntw.s[:,1,0], S21)
        npy.testing.assert_array_almost_equal(ntw.s[:,1,1], S11)

    def test_s_lossless_line(self):
        '''
        Lossless transmission line of characteristic impedance z1, length l
        and wavenumber beta
              _______
        ○-----       -----○
          z0     z1    z0
        ○-----_______-----○

        '''
        l = 5.0
        z1 = 30.0
        z0 = self.dummy_media.z0

        ntw = self.dummy_media.line(d=0, unit='m', z0=z0) \
            ** self.dummy_media.line(d=l, unit='m', z0=z1) \
            ** self.dummy_media.line(d=0, unit='m', z0=z0)

        beta = self.dummy_media.beta
        _z1 = z1/z0
        S11 = 1j*(_z1**2 - 1)*npy.sin(beta*l) / \
            (2*_z1*npy.cos(beta*l) + 1j*(_z1**2 + 1)*npy.sin(beta*l))
        S21 = 2*_z1 / \
            (2*_z1*npy.cos(beta*l) + 1j*(_z1**2 + 1)*npy.sin(beta*l))
        npy.testing.assert_array_almost_equal(ntw.s[:,0,0], S11)
        npy.testing.assert_array_almost_equal(ntw.s[:,0,1], S21)
        npy.testing.assert_array_almost_equal(ntw.s[:,1,0], S21)
        npy.testing.assert_array_almost_equal(ntw.s[:,1,1], S11)

    def test_s_lossy_line(self):
        '''
        Lossy transmission line of characteristic impedance Z0, length l
        and propagation constant gamma = alpha + j beta

        ○---------○

        ○---------○

        has ABCD matrix of the form:

        [ cosh(gamma l)       Z0 sinh(gamma l) ]
        [ 1/Z0 sinh(gamma l)  cosh(gamma l) ]
        '''


class ABCDTwoPortsNetworkTestCase(unittest.TestCase):
    '''
    Check that ABCD parameters of media base elements (such as lumped elements)
    versus theoretical results.
    '''
    def setUp(self):
        self.dummy_media = DefinedGammaZ0(
            frequency=Frequency(1, 100, 21,'GHz'),
            gamma=1j,
            z0=50 ,
            )

    def test_abcd_series_element(self):
        '''
        Series elements of impedance Z:

        ○---[Z]---○

        ○---------○

        have ABCD matrix of the form:
        [ 1  Z ]
        [ 0  1 ]
        '''
        R = 1.0  # Ohm
        ntw = self.dummy_media.resistor(R)
        npy.testing.assert_array_almost_equal(ntw.a[:,0,0], 1.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,0,1], R)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,0], 0.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,1], 1.0)

    def test_abcd_shunt_element(self):
        '''
        Shunt elements of admittance Y:

        ○---------○
            |
           [Y]
            |
        ○---------○

        have ABCD matrix of the form:
        [ 1  0 ]
        [ Y  1 ]
        '''
        R = 1.0  # Ohm
        ntw = self.dummy_media.shunt(self.dummy_media.resistor(R)**self.dummy_media.short())
        npy.testing.assert_array_almost_equal(ntw.a[:,0,0], 1.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,0,1], 0.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,0], 1.0/R)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,1], 1.0)

    def test_abcd_series_shunt_elements(self):
        '''
        Series and Shunt elements of impedance Zs and Zp:

        ○---[Zs]--------○
                 |
                [Zp]
                 |
        ○--------------○
        have ABCD matrix of the form:
        [ 1 + Zs/Zp    Zs ]
        [ 1/Zp          1 ]
        '''
        Rs = 2.0
        Rp = 3.0
        serie_resistor = self.dummy_media.resistor(Rs)
        shunt_resistor = self.dummy_media.shunt(self.dummy_media.resistor(Rp) ** self.dummy_media.short())

        ntw = serie_resistor ** shunt_resistor

        npy.testing.assert_array_almost_equal(ntw.a[:,0,0], 1.0+Rs/Rp)
        npy.testing.assert_array_almost_equal(ntw.a[:,0,1], Rs)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,0], 1.0/Rp)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,1], 1.0)

    def test_abcd_thru(self):
        '''
        Thru has ABCD matrix of the form:
        [ 1  0 ]
        [ 0  1 ]
        '''
        ntw = self.dummy_media.thru()
        npy.testing.assert_array_almost_equal(ntw.a[:,0,0], 1.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,0,1], 0.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,0], 0.0)
        npy.testing.assert_array_almost_equal(ntw.a[:,1,1], 1.0)

    def test_abcd_lossless_line(self):
        '''
        Lossless transmission line of characteristic impedance Z0, length l
        and wavenumber beta

        ○---------○

        ○---------○

        has ABCD matrix of the form:

        [ cos(beta l)       j Z0 sin(beta l) ]
        [ j/Z0 sin(beta l)  cos(beta l) ]
        '''
        l = 5
        z0 = 80
        ntw = self.dummy_media.line(d=l, unit='m', z0=z0)
        beta = self.dummy_media.beta
        npy.testing.assert_array_almost_equal(ntw.a[:,0,0], npy.cos(beta*l))
        npy.testing.assert_array_almost_equal(ntw.a[:,0,1], 1j*z0*npy.sin(beta*l))
        npy.testing.assert_array_almost_equal(ntw.a[:,1,0], 1j/z0*npy.sin(beta*l))
        npy.testing.assert_array_almost_equal(ntw.a[:,1,1], npy.cos(beta*l))

    def test_abcd_lossy_line(self):
        '''
        Lossy transmission line of characteristic impedance Z0, length l
        and propagation constant gamma = alpha + j beta

        ○---------○

        ○---------○

        has ABCD matrix of the form:

        [ cosh(gamma l)       Z0 sinh(gamma l) ]
        [ 1/Z0 sinh(gamma l)  cosh(gamma l) ]
        '''
        l = 5.0
        z0 = 30.0
        alpha = 0.5
        beta = 2.0
        lossy_media = DefinedGammaZ0(
            frequency=Frequency(1, 100, 21, 'GHz'),
            gamma=alpha + 1j*beta,
            z0=z0
            )
        ntw = lossy_media.line(d=l, unit='m', z0=z0)
        gamma = lossy_media.gamma
        npy.testing.assert_array_almost_equal(ntw.a[:,0,0], npy.cosh(gamma*l))
        npy.testing.assert_array_almost_equal(ntw.a[:,0,1], z0*npy.sinh(gamma*l))
        npy.testing.assert_array_almost_equal(ntw.a[:,1,0], 1.0/z0*npy.sinh(gamma*l))
        npy.testing.assert_array_almost_equal(ntw.a[:,1,1], npy.cosh(gamma*l))
